' Copy this code to the Click event of a button
' Name the button - cmdRectPattern
' Solid Edge should be running with no document open
' Tested with Solid Edge 7 only - They dropped the objModel.RectangularPatterns.Add 
' method with SolidEdge 10

Private Sub cmdRecPattern_Click()
    Dim objApp As Object
    Dim objDoc As Object
    Dim objRefPlane As Object
    Dim objProfile As Object
    Dim objProfiles(1 To 2) As Object
    Dim dblX As Double
    Dim dblY As Double
    Dim ordStatus As Long
    Dim objModel As Object
    Dim objCutout As Object
    Dim objFeatures(1 To 2) As Object
    Dim lngStatus As Long
    
    
    ' Connect to a running instance of Solid Edge.
    Set objApp = GetObject(, "SolidEdge.Application")

    
    ' Create a new Part document.
    Set objDoc = objApp.Documents.Add(ProgID:="SolidEdge.PartDocument")
    
    ' Create a new reference plane.
    Set objRefPlane = objDoc.RefPlanes.AddParallelByDistance( _
                            ParentPlane:=objDoc.RefPlanes(1), _
                            distance:=0, _
                            NormalSide:=igLeft, _
                            Local:=True)
                            
    ' Create a new profile to use for the feature.

    Set objProfiles(1) = objDoc.ProfileSets.Add.Profiles.Add(objRefPlane)
    
    ' Get the model space origin relative to profile space.
    Call objProfiles(1).Convert3DCoordinate(x3d:=0, _
                                    y3d:=0, z3d:=0, _
                                    x2d:=dblX, y2d:=dblY)
    
    ' Draw a circle for the profile graphics.
    Call objProfiles(1).Circles2d.AddByCenterRadius(_
                        x:=dblX, y:=dblY, Radius:=0.1)


    ' End and validate the profile.
    lngStatus = objProfiles(1).End(ValidationCriteria:=igProfileClosed)
    If lngStatus <> 0 Then
        MsgBox "Invalid Profile"
        Exit Sub
    End If
    
    ' Create the base protrusion feature.
    Set objModel = objDoc.Models.AddFiniteExtrudedProtrusion(NumberOfProfiles:=1, _
                                    ProfileArray:=objProfiles, _
                                    ProfilePlaneSide:=igLeft, _

                                    ExtrusionDistance:=0.01)
                                            
    ' Turn off the display of the profile
    objProfiles(1).Visible = False
    
    ' Create a new reference plane for a small cutout feature.
    Set objRefPlane = objDoc.RefPlanes.AddParallelByDistance( _
                            ParentPlane:=objDoc.RefPlanes(1), _
                            distance:=0, _
                            NormalSide:=igLeft, _

                            Local:=True)
    
    ' Create a new profile to use for the feature.
    Set objProfile = objDoc.ProfileSets.Add.Profiles.Add(objRefPlane)
    
    ' Get the model space origin relative to profile space.
    Call objProfile.Convert3DCoordinate(x3d:=0, y3d:=0, z3d:=0, _
                                        x2d:=dblX, y2d:=dblY)
                                            
    ' Draw a circle for the profile graphics.

    Call objProfile.Circles2d.AddByCenterRadius(x:=dblX - 0.065, y:=dblY - 0.065, Radius:=0.005)

    ' End and validate the profile.
    ordStatus = objProfile.End(ValidationCriteria:=igProfileClosed)
    If ordStatus <> 0 Then
        MsgBox "Invalid Profile"
        Exit Sub
    End If
    
    ' Turn off the display of the profile
    objProfile.Visible = False
    
    ' Create the cutout feature.
    Set objCutout = objModel.ExtrudedCutouts.AddFinite(Profile:=objProfile, _

                                   profileSide:=igLeft, _
                                   ProfilePlaneSide:=igLeft,_
                                   Depth:=0.003)
    
    ' Create a reference plane for the pattern.
    Set objRefPlane = objDoc.RefPlanes.AddParallelByDistance( _
                            ParentPlane:=objDoc.RefPlanes(1), _
                            distance:=0, _
                            NormalSide:=igLeft, _
                            Local:=True)

    
    ' Build up the array of features to pattern.  In this case there's only one feature.
    Set objFeatures(1) = objCutout
    
    Call objModel.RectangularPatterns.Add( _
                             PatternPlane:=objRefPlane, _
                             NumberOfFeatures:=1, _
                             featureArray:=objFeatures, _
                             XCount:=3, YCount:=3, _
                             xOffset:=0.065, _
                             yOffset:=0.065, Angle:=0)

End Sub
